@@ -403,6 +403,7 @@ class Instance(Base):
             raise ServiceForbidden()
 
         user_attributes = [attr.strip() for attr in ucr.get('self-service/udm_attributes', '').split(',')]
+
         user = self.get_udm_user_by_dn(dn)
         user.set_defaults = True
         user.set_default_values()
@@ -418,7 +419,7 @@ class Instance(Base):
 
     def _get_user_attributes_descriptions(self):
         user_attributes = [attr.strip() for attr in ucr.get('self-service/udm_attributes', '').split(',')]
-
+        read_only_attributes = [attr.strip() for attr in ucr.get('self-service/udm_attributes/read-only', '').split(',')]
         widget_descriptions = []
         label_overwrites = {
             'jpegPhoto': _('Your picture'),
@@ -437,7 +438,7 @@ class Instance(Base):
                 'size': prop.size or prop.syntax.size,
                 'required': bool(prop.required),
                 'editable': bool(prop.may_change),
-                'readonly': not bool(prop.editable),
+                'readonly': not bool(prop.editable) or propname in read_only_attributes,
                 'multivalue': bool(prop.multivalue),
             }
             widget_description.update(prop.syntax.get_widget_options(prop))
@@ -563,6 +564,12 @@ class Instance(Base):
             raise ServiceForbidden()
 
         user_attributes = [attr.strip() for attr in ucr.get('self-service/udm_attributes', '').split(',')]
+        read_only_attributes = [attr.strip() for attr in ucr.get('self-service/udm_attributes/read-only', '').split(',')]
+
+        for attr in attributes:
+            if attr in read_only_attributes:
+                MODULE.error('set_user_attributes(): attribute %s is read-only' % (attr,))
+                raise UMC_Error(_('The attribute %s is read-only.') % (attr,))
         user = self.usersmod.object(None, lo, po, dn)
         user.open()
         for propname, value in attributes.items():
@@ -575,6 +582,12 @@ class Instance(Base):
             raise UMC_Error(_('The attributes could not be saved: %s') % (UDM_Error(exc)))
         return _("Successfully changed your profile data.")
 
+    def _get_password_complexity_message(self):
+        return ucr.get(
+            'umc/login/password-complexity-message/%s' % (self.locale.language,),
+            ucr.get('umc/login/password-complexity-message/en', ''),
+        )
+
     @forward_to_master
     @simple_response
     def create_self_registered_account(self, attributes):
@@ -670,11 +683,12 @@ class Instance(Base):
         try:
             new_user.create()
         except univention.admin.uexceptions.base as exc:
-            MODULE.error('create_self_registered_account(): could not create user: %s' % (traceback.format_exc(),))
+            password_complexity_message = self._get_password_complexity_message() if isinstance(exc, (udm_errors.pwToShort, udm_errors.pwQuality)) else ''
+            MODULE.error('create_self_registered_account(): could not create user: %s' % (exc,))
             return {
                 'success': False,
                 'failType': 'CREATION_FAILED',
-                'data': _('The account could not be created:\n%s') % UDM_Error(exc),
+                'data': (_('The account could not be created:\n%s\n%s') % (UDM_Error(exc), password_complexity_message)).rstrip(),
             }
         finally:
             # TODO cleanup
@@ -1167,7 +1181,8 @@ class Instance(Base):
         try:
             user.modify()
         except (udm_errors.pwToShort, udm_errors.pwQuality) as exc:
-            raise UMC_Error(str(exc))
+            password_complexity_message = ucr.get('umc/login/password-complexity-message/%s' % (self.locale.language,), ucr.get('umc/login/password-complexity-message/en', ''))
+            raise UMC_Error(("%s %s" % (exc, password_complexity_message)).rstrip())
         except udm_errors.pwalreadyused as exc:
             raise UMC_Error(exc.message)
         except Exception:
